@@ -8,26 +8,28 @@ import numpy as np
 import pandas as pd
 from tqdm import tqdm
 
-from . import AUDIO, VIDEO
+from . import AUDIO, VIDEO, MISSING_TIMEZONE_FORCED_TIMEZONE, MISSING_TIMEZONE_RECORDING_IDS
 from .cha import export_cha_to_csv
 from .codebooks import make_codebook_template
 from .gather import gather_all_basic_level_annotations, write_all_basic_level_to_csv, write_all_basic_level_to_feather, \
     check_for_errors
 from .io import read_global_basic_level, blab_write_csv, blab_read_csv, SEEDLINGS_NOUNS_DTYPES, SEEDLINGS_NOUNS_SORT_BY, \
-    SEEDLINGS_NOUNS_REGIONS_DTYPES, SEEDLINGS_NOUNS_SUB_RECORDINGS_DTYPES, SEEDLINGS_NOUNS_RECORDINGS_DTYPES
+    SEEDLINGS_NOUNS_REGIONS_DTYPES, SEEDLINGS_NOUNS_SUB_RECORDINGS_DTYPES, SEEDLINGS_NOUNS_RECORDINGS_DTYPES, \
+    read_video_recordings_csv, read_seedlings_codebook
 from .listened_time import listen_time_stats_for_report, _get_subregion_count, _preprocess_region_info, RegionType
 from .merge import create_merged, FIXME
 from .opf import export_opf_to_csv
 from .paths import get_all_opf_paths, get_all_cha_paths, get_basic_level_path, _parse_out_child_and_month, \
     ensure_folder_exists_and_empty, _check_modality, get_seedlings_path, get_cha_path, get_opf_path, \
-    _normalize_child_month, get_lena_5min_csv_path, get_its_path, split_recording_id
+    _normalize_child_month, get_lena_5min_csv_path, get_its_path, split_recording_id, get_seedlings_nouns_private_path
 from .regions import get_processed_audio_regions as _get_processed_audio_regions, _get_amended_regions, \
     SPECIAL_CASES as AUDIO_SPECIAL_CASES, get_top3_top4_surplus_regions as _get_top3_top4_surplus_regions, \
     are_tokens_in_top3_top4_surplus
 # Placeholder value for words without the basic level information
 from .regions.regions import calculate_total_listened_time_ms, calculate_total_recorded_time_ms
 from .scatter import copy_all_basic_level_files_to_subject_files
-from ..its import Its
+from .. import ANONYMIZATION_DATE
+from ..its import Its, ItsNoTimeZoneInfo
 
 
 def export_all_opfs_to_csv(output_folder: Path, suffix='_processed'):
@@ -560,25 +562,41 @@ def get_top3_top4_surplus_regions(subject, month):
     return _get_top3_top4_surplus_regions(processed_regions, month)
 
 
-def get_lena_recordings(subject, month):
+def get_lena_recordings(recording_id):
     """
-    Get anonymized information about sub-recordings of the LENA recording for the given subject and month.
+    Get anonymized information about sub-recordings of the LENA recording for the given subject and month. There are
+    several special cases of recordings that are missing an its file or that have one but it doesn't contain the
+    timezone information. See __init__.py for details.
 
-    If there are pauses in LENA recordings, LENA still outputs one big wav file. This leads to confusion and errors, such
-    as when an interval spans multiple recordings.
+    If there are pauses in LENA recordings, LENA still outputs one big wav file. This leads to confusion and errors,
+    such as when an interval spans multiple recordings. It is important to know about these pauses.
 
-    :param subject: int/str, subject id
-    :param month: int/str, month
-    :return: a pandas dataframe with the LENA sub-recordings
+    :param recording_id: full recording id, e.g. 'Video_01_16'
+    :return: a pandas dataframe with the LENA sub-recordings and the total duration of all sub-recordings
     """
-    its = Its.from_path(get_its_path(subject, month))
-    recordings = its.gather_recordings(anonymize=True)
+    # TODO: figure out why the timezone info is missing
+    if recording_id in MISSING_TIMEZONE_RECORDING_IDS:
+        forced_timezone = MISSING_TIMEZONE_FORCED_TIMEZONE
+    else:
+        forced_timezone = None
+    subject, month = recording_id.split('_')[1:3]
+    its = Its.from_path(get_its_path(subject, month), forced_timezone=forced_timezone)
 
-    # In a table with recordings only, it doesn't make sense to have each column to start with 'recording_', the way
-    # Its.gather_recordings() does.
-    recordings.columns = recordings.columns.str.replace('recording_', '')
+    try:
+        recordings = its.gather_recordings(anonymize=True)
+    except ItsNoTimeZoneInfo as e:
+        raise ItsNoTimeZoneInfo(f'No timezone info for {subject}_{month}. Please force a timezone using'
+                                f'`forced_timezone`') from e
 
-    return recordings
+    # Change column names to match what `calculate_total_recorded_time_ms` expects
+    recordings = recordings.rename(
+        columns={'recording_start': 'start',
+                 'recording_end': 'end',
+                 'recording_start_wav': 'start_position_ms'},
+        errors='raise')
+    total_recorded_time = calculate_total_recorded_time_ms(recordings=recordings)
+
+    return recordings, total_recorded_time
 
 
 def _sort_tokens(tokens_df):
@@ -587,29 +605,20 @@ def _sort_tokens(tokens_df):
     :param tokens_df: a dataframe to sort
     :return: sorted dataframe
     """
-    sort_by = SEEDLINGS_NOUNS_SORT_BY['seedlings-nouns.csv']
-    # The only difference between dataframe is subj/child
+    sort_by = SEEDLINGS_NOUNS_SORT_BY['seedlings-nouns.csv'].copy()
+    # The only difference between the three dataframes is subj/child
     if 'subj' in tokens_df.columns:
         sort_by[sort_by.index('child')] = 'subj'
     return tokens_df.sort_values(sort_by).reset_index(drop=True)
 
 
-# TODO: return dict, returning a heterogeneous tuple has and will lead to errors
-def gather_recording_seedlings_nouns(recording_id, global_basic_level_for_recording):
+# TODO: return namedtuple, returning a heterogeneous tuple has and will lead to errors
+def gather_recording_nouns_audio(subject, month, global_basic_level_for_recording):
     """
-    Gathers all the data needed to update seedlings_nouns for one recording. For video, the global basic level data
-    is all there is. For audio, we will need to additionally:
+    Gathers all the data needed to update seedlings_nouns for one audio recording.
 
-    - gather processed subregions (listened to parts only) and top3/top4/surplus regions (much of the time will be
-      listed multiple times)
-    - add top3/top4/surplus status for each token to global_basic_level_for_recording,
-    - gather LENA sub-recordings info with time of day and time within the wav file
-    - calculate total recorded time,
-    - calculate total listened time.
-
-    To update seedlings_nouns, we'll just concatenate all of these.
-
-    :param recording_id: full recording id, e.g. 'Video_01_16'
+    :param subject: subject id, e.g. '01'
+    :param month: month, e.g. '08'
     :param global_basic_level_for_recording: the rows of global_basic_level that correspond to the recording
     :return: (top3/top4/surplus regions,
               global_basic_level_for_recording with is_top3/is_top4/is_surplus added,
@@ -617,13 +626,6 @@ def gather_recording_seedlings_nouns(recording_id, global_basic_level_for_record
               total recorded time,
               total listened time)
     """
-    modality, subject, month = split_recording_id(recording_id)
-
-    if modality == VIDEO:
-        # We want to return the same number of items, hence the None's
-        return (global_basic_level_for_recording,) + (None,) * 4
-
-    # modality == AUDIO
     # Regions
     processed_regions = get_processed_audio_regions(subject, month, amend_if_special_case=True)
     top3_top4_surplus_regions = _get_top3_top4_surplus_regions(processed_regions=processed_regions, month=month)
@@ -643,26 +645,7 @@ def gather_recording_seedlings_nouns(recording_id, global_basic_level_for_record
                                  .pipe(_sort_tokens))
 
     # Sub-recordings and time totals
-    # TOD0:
-    # - move the list to blabpy.seedlings so that it isn't hidden in the code,
-    # - find 01_08.its and remove this bodge,
-    # - figure out why the other ones are missing timezone info.
-    problem_recordings_total_recorded_time = {'Audio_01_08': 57600125,  # missing its file
-                                              'Audio_12_17': 57600000,  # missing timezone info
-                                              'Audio_18_09': 43822125,  # missing timezone info
-                                              'Audio_20_13': 57600125,  # missing timezone info
-                                              'Audio_26_13': 57600125,  # missing timezone info
-                                              'Audio_29_16': 44096000,  # missing timezone info
-                                              }
-
-    if recording_id in problem_recordings_total_recorded_time:
-        lena_recordings = pd.DataFrame(
-            data=dict(start=[None], end=[None], start_position_ms=[None])).astype(
-            dtype=dict(start=np.datetime64, end=np.datetime64, start_position_ms=pd.Int64Dtype()))
-        total_recorded_time = problem_recordings_total_recorded_time[recording_id]
-    else:
-        lena_recordings = get_lena_recordings(subject, month).rename(columns={'start_wav': 'start_position_ms'})
-        total_recorded_time = calculate_total_recorded_time_ms(recordings=lena_recordings)
+    lena_recordings, total_recorded_time = get_lena_recordings(recording_id=f'{AUDIO}_{subject}_{month}')
     total_listened_time = calculate_total_listened_time_ms(processed_regions=processed_regions, month=month,
                                                            recordings=lena_recordings)
 
@@ -688,6 +671,83 @@ def gather_recording_seedlings_nouns(recording_id, global_basic_level_for_record
             total_recorded_time)
 
 
+def load_video_recordings_csv(anonymize=True):
+    """
+    Loads the video recordings csv from PN-OPUS.
+    :return: a dataframe with the video recordings
+    """
+    video_info_df = read_video_recordings_csv()
+
+    # Change start dates to happen on the same date. Shift end dates by the same deltas.
+    if anonymize:
+        deltas = video_info_df.start.dt.date - ANONYMIZATION_DATE
+        video_info_df = video_info_df.assign(start=lambda df: df.start - deltas,
+                                             end=lambda df: df.end - deltas)
+
+    return video_info_df
+
+
+def get_video_times(subject, month):
+    """
+    Gets the time of day of start and end, and duration times for a given subject and month.
+
+    :param subject: subject id, e.g. '01'
+    :param month: month, e.g. '08'
+    :return: start_dt, end_dt, duration_ms
+    """
+    subject, month = _normalize_child_month(child=subject, month=month)
+    subject_month = f'{subject}_{month}'
+    video_recordings_info = load_video_recordings_csv(anonymize=True)
+    ((start_dt, end_dt, duration),) = video_recordings_info.loc[lambda df: df.subject_month == subject_month,
+                                                                ['start', 'end', 'duration']].values
+    duration_ms = duration.total_seconds() * 1000
+
+    return start_dt, end_dt, duration_ms
+
+
+def gather_recording_nouns_video(subject, month, global_basic_level_for_recording):
+    """
+    Gathers all the data needed to update seedlings_nouns for one video recording.
+
+    :param subject: subject id, e.g. '01'
+    :param month: month, e.g. '08'
+    :param global_basic_level_for_recording: the rows of global_basic_level that correspond to the recording
+    :return: (global_basic_level_for_recording with is_top3/is_top4/is_surplus added,
+              top3/top4/surplus regions (None because video),
+              sub-recordings (a single one because video),
+              total recorded time,
+              total listened time)
+    """
+    start_dt, end_dt, duration_ms = get_video_times(subject, month)
+    sub_recordings_df = pd.DataFrame.from_dict(dict(start=[start_dt],
+                                               end=[end_dt],
+                                               start_position_ms=[0]))
+
+    return global_basic_level_for_recording, None, sub_recordings_df, duration_ms, duration_ms
+
+
+def gather_recording_seedlings_nouns(recording_id, global_basic_level_for_recording):
+    """
+    Gathers all the data needed to update seedlings_nouns for one recording.
+
+    :param recording_id: full recording id, e.g. 'Video_01_16'
+    :param global_basic_level_for_recording: the rows of global_basic_level that correspond to the recording
+    :return: (top3/top4/surplus regions,
+              global_basic_level_for_recording with is_top3/is_top4/is_surplus added,
+              LENA recordings,
+              total recorded time,
+              total listened time)
+    """
+    modality, subject, month = split_recording_id(recording_id)
+
+    if modality == VIDEO:
+        return gather_recording_nouns_video(subject, month, global_basic_level_for_recording)
+    elif modality == AUDIO:
+        return gather_recording_nouns_audio(subject, month, global_basic_level_for_recording)
+    else:
+        raise ValueError(f'Unknown modality {modality} for recording {recording_id}')
+
+
 def _preprocess_global_basic_level(global_basic_level):
     """
     Preprocess global basic level for seedlings_nouns
@@ -709,14 +769,15 @@ def _preprocess_global_basic_level(global_basic_level):
     return global_basic_level_preprocessed
 
 
-def _gather_corpus_seedlings_nouns(global_basiclevel_path):
+def _gather_corpus_seedlings_nouns(global_basiclevel_df):
     """
     Create all the csv for the seedlings_nouns dataset
-    :param global_basiclevel_path: path to global_basiclevel.csv
+    :param global_basiclevel_df: a global basic level dataframe (all_basiclevel + global_bl)
     :return: None
     """
+    gbl = _preprocess_global_basic_level(global_basic_level=global_basiclevel_df)
+
     # Gather data for each recording and put into lists
-    gbl = (read_global_basic_level(global_basiclevel_path).pipe(_preprocess_global_basic_level))
     print('Processing video recordings is instantaneous, only audio recordings take time.')
     everything = [
         (recording_id,) +
@@ -786,14 +847,12 @@ def _make_updated_codebook(df, old_codebook_path):
     codebook_template = make_codebook_template(df)
 
     # Merge with the old codebook
-    is_new_dataframe, has_new_columns = False, False
     if old_codebook_path.exists():
         codebook_template = codebook_template.drop(columns='description')
         # Load manually update columns from the old codebook
         auto_generated_columns = set(codebook_template.columns)
-        old_codebook = blab_read_csv(old_codebook_path).drop(columns=auto_generated_columns - {'column'})
         codebook_template = codebook_template.merge(
-            blab_read_csv(old_codebook_path).drop(columns=auto_generated_columns - {'column'}),
+            read_seedlings_codebook(old_codebook_path).drop(columns=auto_generated_columns - {'column'}),
             on='column', how='left')
 
     n_vars_without_description = codebook_template.description.isna().sum()
@@ -855,14 +914,14 @@ def _print_seedlings_nouns_update_instructions(new_dataframes, new_variables,
     print('0. If there are warnings above about new dataframes or new variables, update descriptions in the'
           f' corresponding codebooks in the following folder:\n{new_seedlings_nouns_dir}\n\n'
           '1. Copy the csv files\n'
-          f'from: {new_seedlings_nouns_dir}\n'
-          f'to:   {old_seedlings_nouns_dir}\n\n'
+          f'from: {new_seedlings_nouns_dir.absolute()}\n'
+          f'to:   {old_seedlings_nouns_dir.absolute()}\n\n'
           '2. Go to\n'
-          f'{old_seedlings_nouns_dir}\n'
-          'and follow the instructions in CONTRIBUTING.md to update the dataset.\n')
+          f'{old_seedlings_nouns_dir.absolute()}\n'
+          'and follow the instructions in CONTRIBUTING.md to finish updating the dataset.\n')
 
 
-def make_updated_seedlings_nouns(global_basiclevel_path, seedlings_nouns_dir, output_dir=Path()):
+def _make_updated_seedlings_nouns(global_basiclevel_path, seedlings_nouns_dir, output_dir=Path()):
     """
     Write all the csvs and their codebooks to a given folder based on a csv with global basic level data and a folder
     that contains existing codebooks.
@@ -874,7 +933,8 @@ def make_updated_seedlings_nouns(global_basiclevel_path, seedlings_nouns_dir, ou
     ensure_folder_exists_and_empty(output_dir)
 
     # Gather and write data
-    seedlings_nouns, regions, sub_recordings, recordings = _gather_corpus_seedlings_nouns(global_basiclevel_path)
+    global_basiclevel_path_df = read_global_basic_level(global_basiclevel_path)
+    seedlings_nouns, regions, sub_recordings, recordings = _gather_corpus_seedlings_nouns(global_basiclevel_path_df)
 
     new_dataframes = []
     new_variables = []
@@ -893,3 +953,24 @@ def make_updated_seedlings_nouns(global_basiclevel_path, seedlings_nouns_dir, ou
     _print_seedlings_nouns_update_instructions(new_dataframes=new_dataframes, new_variables=new_variables,
                                                new_seedlings_nouns_dir=output_dir,
                                                old_seedlings_nouns_dir=seedlings_nouns_dir)
+
+
+def make_updated_seedlings_nouns():
+    """Creates updated versions of all seedlings-nouns csvs.
+
+    For this function to work, the following must be true:
+
+    - You are connected to PN-OPUS.
+    - Current working directory contains `global-basic-level.csv` with updated global basic level version.
+      It can be created using `blabr:::make_new_global_basic_level()`.
+    - There is no subfolder `new_csvs` in the current folder or it is empty.
+    """
+    global_basiclevel_path = Path('global-basic-level.csv')
+    assert global_basiclevel_path.exists(), (f'File {global_basiclevel_path.name} does not exist in the current working'
+                                             f' directory.')
+    # get_*_path functions check that paths exist - we don't need to do it here
+    seedlings_nouns_dir = get_seedlings_nouns_private_path() / 'public'
+    output_dir = Path('new_csvs')
+    ensure_folder_exists_and_empty(output_dir)
+
+    _make_updated_seedlings_nouns(global_basiclevel_path, seedlings_nouns_dir, output_dir)
